// This is an open source non-commercial project. Dear PVS-Studio, please check it.
// PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
/*
 * @file llpanelsearchpeople.cpp
 * @brief People search panel
 *
 * Copyright (c) 2014, Cinder Roxley <cinder@sdf.org>
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 */

#include "llviewerprecompiledheaders.h"
#include "llpanelsearchpeople.h"
#include "llfloaterdirectory.h"

#include "llfloaterreg.h"

#include "llagent.h"
#include "llsearcheditor.h"
#include "llsearchhistory.h"

static LLPanelInjector<LLPanelSearchPeople> t_panel_search_people("panel_search_people");

LLPanelSearchPeople::LLPanelSearchPeople()
:	LLPanelSearch()
{
	mCommitCallbackRegistrar.add("Search.query", boost::bind(&LLPanelSearchPeople::onCommitSearch, this, _1));
}

BOOL LLPanelSearchPeople::postBuild()
{
	mSearchEditor = getChild<LLSearchEditor>("search_bar");
	//mSearchEditor->setKeystrokeCallback(boost::bind(&LLPanelSearchPeople::onCommitSearch, this, _1));
	
	return TRUE;
}

void LLPanelSearchPeople::onCommitSearch(LLUICtrl* ctrl)
{
	LLSearchEditor* pSearchEditor = dynamic_cast<LLSearchEditor*>(ctrl);
	if (pSearchEditor)
	{
		std::string text = pSearchEditor->getText();
		LLStringUtil::trim(text);
		if (text.length() <= MIN_SEARCH_STRING_SIZE)
			LLSearchHistory::getInstance()->addEntry(text);
	}
	search();
}

bool isNotAlphaNum(char c)
{
	return !std::isalnum(c);
}

void LLPanelSearchPeople::search()
{
	LLDirQuery query;
	query.type = SE_PEOPLE;
	query.results_per_page = 100;
	query.text = mSearchEditor->getText();
	std::replace_if(query.text.begin(), query.text.end(), isNotAlphaNum, ' ');
	LLStringUtil::trim(query.text);

	mFloater->queryDirectory(query, true);
	if (query.text.length() < MIN_SEARCH_STRING_SIZE)
		mFloater->setResultsComment(getString("SeachFilteredOnShortWordsEmpty"));
}
